<?php

/**
 * @file
 * Provides general pages and UI functions for the PIFT module.
 */

/**
 * Add additional description to file attachment form on issues.
 *
 * Invoked from within pift_form_project_issue_node_form_alter(), to modify the
 * file upload field description text.
 *
 * @see pift_form_project_issue_node_form_alter()
 */
function pift_pages_upload_description_add(&$form, $form_state, $form_id) {
  // Add description on field_issue_files if it is set.
  if (PIFT_DESCRIPTION) {
    $nid = $form['#node']->field_project[LANGUAGE_NONE][0]['target_id'];
    if (isset($form['field_issue_files'][LANGUAGE_NONE]['#file_upload_description']) && $nid && pift_project_enabled($nid)) {
      $description = filter_xss_admin(PIFT_DESCRIPTION);
      $form['field_issue_files'][LANGUAGE_NONE]['#file_upload_description'] .= " " . $description;
    }
  }
}

/**
 * Form constructor for the project testing preferences form on a project.
 *
 * @param object $node
 *   The fully loaded project node object.
 *
 * @see pift_pages_project_issue_settings_submit().
 */
function pift_pages_project_issue_settings($form, &$form_state, $node) {
  $form = array();
  $form['testing'] = array(
    '#type' => 'fieldset',
    '#title' => t('Testing'),
    '#collapsible' => TRUE,
    '#weight' => -9,
    '#access' => user_access('pift enable project testing'),
  );

  // Build an array of core capatible version strings (ie 6.x, 7.x, etc).
  $api_versions = pift_core_api_versions();
  $versions = array();
  foreach ($api_versions as $api_version) {
    $term = taxonomy_term_load($api_version);
    $versions[] = $term->name;
  }
  $form['testing']['pift_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable automated testing'),
    '#description' => t('Enable automated testing of @versions compatible branches and patches.',
      array('@versions' => implode(', ', $versions))),
    '#default_value' => pift_project_enabled($node->nid),
  );
  $form['testing']['pid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );
  $form['testing']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Settings'),
  );
  return $form;
}

/**
 * Form submission handler for the pift_pages_project_issue_settings() form.
 *
 * Enables or disables testing for a given project.
 */
function pift_pages_project_issue_settings_submit($form, &$form_state) {
  $enabled = pift_project_enabled($form_state['values']['pid']);
  if ($form_state['values']['pift_enable'] && !$enabled) {
    pift_project_enable_testing($form_state['values']['pid']);
    drupal_set_message("Automated testing enabled.");
  }
  else if (!$form_state['values']['pift_enable'] && $enabled) {
    pift_project_disable_testing($form_state['values']['pid']);
    drupal_set_message("Automated testing disabled.");
  }
}

/**
 * Form confirmation page for the 're-test' link associated with a file.
 *
 * @param integer $test_id
 *   The unique test ID to be re-tested.
 */
function pift_pages_retest_confirm_form($form, &$form_state, $test_id) {
  $test = pift_test_get_summary($test_id);

  if (!empty($test)) {
    $cid = pift_test_get_comment_id($test);
    $form = array();
    // Key off type to determine if this is a file retest or branch retest
    if ($test['type'] == PIFT_TYPE_FILE) {
      $file = file_load($test['id']);
      // File test
      $form['test'] = array(
        '#type' => 'value',
        '#value' => array(
          'test_id' => $test_id,
          'fid' => $test['id'],
          'nid' => $test['nid'],
          'cid' => $cid,
          'status' => $test['status'],
          'title' => $file->filename,
          'type' => PIFT_TYPE_FILE,
        ),
      );
      $form_state['#redirect'] = 'node/' . $form['test']['#value']['nid'];
    }
    elseif ($test['type'] == PIFT_TYPE_BRANCH) {
      // Branch test
      $form['test'] = array(
        '#type' => 'value',
        '#value' => array(
          'test_id' => $test_id,
          'status' => $test['status'],
          'title' => "Test " . $test['test_id'],
          'type' => PIFT_TYPE_BRANCH,
        ),
      );
      // Get the project id from the release nid
      $pid = pift_test_get_pid($test);
      $form_state['#redirect'] = 'node/' . $pid;
    }
    return confirm_form(
      $form,
      t('Are you sure you want to request that %title be re-tested?', array('%title' => $form['test']['#value']['title'])),
      $form_state['#redirect'],
      t('Once requested the test will be re-queued and results will be updated once the re-test has finished.'),
      t('Re-test'),
      t('Cancel')
    );
  }
  else {
    drupal_set_message(t('Invalid test.'), 'error');
  }
}

/**
 * Form submission handler for the pift_pages_retest_confirmation_form form.
 *
 * Requeue test if it still meets our criteria.
 */
function pift_pages_retest_confirm_form_submit($form, &$form_state) {
  global $user;
  $test = $form_state['values']['test'];
  if ($test['status'] > PIFT_STATUS_SENT) {
    if ($test['type'] == PIFT_TYPE_FILE) {
      // Base changes to be made in followup comment.
      $node = node_load($test['nid']);
      $node->nodechanges_uid = $user->uid;
      // TODO: Deduce appropriate comment format (instead of hardcoded '1')
      $node->nodechanges_comment_body = array(
        'value' => theme('pift_auto_followup', array('type' => 'retest', 'nid' => $test['nid'], 'cid' => $test['cid'], 'filename' => $test['title'])),
        'format' => 1,
      );
      // If node issue status is not already a valid status then set to default
      // retest status, otherwise leave status alone.
      if (!in_array($node->field_issue_status[LANGUAGE_NONE][0]['value'], variable_get('pift_status', array()))) {
        $node->field_issue_status[LANGUAGE_NONE][0]['value'] = PIFT_FOLLOWUP_RETEST;
      }
      // Add followup to issue.
      node_save($node);
    }
    // Update test record to reflect change.
    $result = pift_test_requeue($test['test_id']);

    if (!empty($result)) {
      drupal_set_message(t('%title has been submitted for re-testing. Please be patient while you wait for results.',
                           array('%title' => $test['title'])));
    }
    else {
      drupal_set_message(t('Invalid test.'), 'error');
    }
  }
  else {
    drupal_set_message(t('%title is not currently eligible for re-testing.',
                         array('%title' => $test['title'])), 'error');
  }
  $form_state['redirect'] = $form_state['#redirect'];
}

/**
 * Form confirmation page for a test deletion request.
 *
 * @param integer $test_id
 *   The unique test ID to be deleted.
 */
function pift_pages_delete_test_confirm_form($form, &$form_state, $test_id) {
  $test = pift_test_get_summary($test_id);

  if (!empty($test)) {
    $form = array();
    $form['test'] = array(
      '#type' => 'value',
      '#value' => array(
        'test_id' => $test_id,
        'title' => "Test " . $test_id,
      ),
    );
    // Get the project id
    $pid = pift_test_get_pid($test);

    $form_state['#redirect'] = 'node/' . $pid;
    return confirm_form(
      $form,
      t('Are you sure you want to delete %title?', array('%title' => $form['test']['#value']['title'])),
      $form_state['#redirect'],
      t('This will remove any existing results from the site, and reset the test status.') . '<br />' . t('This can help free up a branch test which is stuck in a "sent" state for long periods of time.'),
      t('Delete Test'),
      t('Cancel')
    );
  }
  else {
    drupal_set_message(t('Invalid test.'), 'error');
  }
}

/**
 * Form submission handler for the pift_pages_delete_confirmation_form form.
 *
 * Delete the test.
 */
function pift_pages_delete_test_confirm_form_submit($form, &$form_state) {
  $test_id = $form_state['values']['test']['test_id'];
  if (!empty($test_id)) {
    pift_test_delete($test_id);
  }
  $form_state['redirect'] = $form_state['#redirect'];
}

/**
 * Process the specified attachment information and return the valid status
 * message and and row CSS class.
 *
 * @param integer $status
 *   Test status, PIFT_STATUS_*.
 * @param boolean $has_message
 *   Indicates whether the attachment has an associated message.
 *
 * @return array
 *   The table cell value and CSS class.
 */
function pift_attachment_process($status, $has_message) {
  $class = '';
  if ($status == PIFT_STATUS_QUEUE && $has_message) {
    $cell = t('Queued for re-testing');
    $class = 'pift-retest';
  }
  else if ($status == PIFT_STATUS_QUEUE) {
    $cell = t('Queued for testing');
  }
  elseif ($status == PIFT_STATUS_SENT) {
    $cell = t('Test request sent');
    $class = 'pift-retest';
  }
  elseif ($status > PIFT_STATUS_SENT) {
    $cell = t('Idle');
    $class = $status == PIFT_STATUS_PASS ? 'pift-pass' : 'pift-fail';
  }
  return array($cell, $class);
}
