<?php

/**
 * @file
 * Provide test utility and database functions for the PIFT module.
 */

/**
 * Determine whether a given test_id is valid.
 *
 * @param integer $test_id
 *   The ID of the test to validate.
 *
 * @return boolean
 *   TRUE if the test exists in {pift_data}, or FALSE if not.
 */
/*
function pift_test_exists($test_id) {
  $exists = db_query('SELECT COUNT(test_id) FROM {pift_data} WHERE test_id = :testid',
    array(':testid' => $test_id))
    ->fetchField();
  return !empty($exists);
}
*/

/**
 * Retrieve basic test info for a given test_id.
 *
 * @param integer $test_id
 *   The ID of the test to retrieve.
 *
 * @return array
 *   An array of test information from the {pift_data} table for the given ID.
 */
function pift_test_get_summary($test_id) {
  $tests = pift_test_get_summary_multiple(array($test_id));
  return $tests[$test_id];
}

/**
 * Retrieve basic test info for a given array of test_ids.
 *
 * @param array $test_ids
 *   An array of unique test identifiers.
 * @param string $filter
 *   An optional filter to apply to the query. Valid values include 'file',
 *   'branch', and 'none'.
 *
 * @return array
 *   A nested array of test information from the {pift_data} table for the
 *   given ID.
 */
function pift_test_get_summary_multiple($test_ids, $filter = 'none') {
  $query = db_select('pift_data', 'pt')
    ->fields('pt')
    ->condition('test_id', $test_ids, 'IN');
  switch ($filter) {
    case 'file':
      $query->condition('type', PIFT_TYPE_FILE, '=');
      break;
    case 'branch':
      $query->condition('type', PIFT_TYPE_BRANCH, '=');
      break;
    case 'none':
    default:
      break;
  }
  return $query->execute()->fetchAllAssoc('test_id', PDO::FETCH_ASSOC);
}

/**
 * Retrieve the test ids for the specified file/branch ids
 *
 * Determines the corresponding test_ids for a given array of branch or file
 * ids, depending on the value of $type
 *
 * @param array $ids
 *   An array of branch or file ids, based on $type
 * @param integer $type
 *   The test type being requested, either PIFT_TYPE_BRANCH or PIFT_TYPE FILE.
 *
 * @return array
 *   An array of associated test_ids for the branch or file ids provided, keyed
 *   by $id.
 */
function pift_test_get_test_ids($ids, $type = PIFT_TYPE_FILE) {
  if (empty($ids)) {
    return FALSE;
  }
  return db_query('SELECT id, test_id FROM {pift_data} t WHERE type = :type AND id IN (:ids)',
    array(':ids' => $ids, ':type' => $type),
    array('fetch' => PDO::FETCH_ASSOC))
    ->fetchAllKeyed();
}

/**
 * Retrieve up to PIFT_XMLRPC_MAX outstanding file or branch tests.
 *
 * @param string $type
 *   The type of tests to retrieve, either 'file' or 'branch'. If set to any
 *   other value, then both file and branch tests will be returned.  Defaults
 *   to 'file'.
 *
 * @return array
 *   An array of {pift_data} test information for each outstanding test, keyed
 *   by test_id, where 'id' represents the file id or branch id (depending on
 *   type) for each test. Result count will be limited to PIFT_XMLRPC_MAX.
 */
function pift_test_get_outstanding_tests($type = 'file') {
  $query = db_select('pift_data', 't', array('fetch' => PDO::FETCH_ASSOC))
    ->fields('t')
    ->condition('t.status', PIFT_STATUS_QUEUE, '=')
    ->range(0, PIFT_XMLRPC_MAX)
    ->orderBy('t.id', 'DESC');
  if ($type == 'file') {
    $query->condition('t.type', PIFT_TYPE_FILE, '=');
  }
  elseif ($type == 'branch') {
    $query->condition('t.type', PIFT_TYPE_BRANCH, '=');
  }
  $result = $query->execute();
  foreach($result as $record) {
    $tests[$record['test_id']] = $record;
  }
  return $tests;
}

/**
 * Determine the project node id for a given test.
 *
 * @param integer $test
 *   The loaded test summary array for this test.
 *
 * @return integer
 *   The node id for the project this test belongs to.
 */
function pift_test_get_pid($test) {
  $node = node_load($test['nid']);
  $language = (isset($issue->language)) ? $issue->language : LANGUAGE_NONE;
  return $issue->field_project[$language][0]['target_id'];
}

/**
 * Sorts an array of tests into file and branch tests.
 *
 * @param array $tests
 *   The array of test information, keyed by test_id and containing key 'type'.
 *
 * @return array
 *   The original arrays test information, sorted into two sub-arrays with keys
 *   'file' and 'branch'.
 */
/*
function pift_test_sort_by_type($tests) {
  $sorted = array('file' => array(), 'branch' => array());
  foreach ($tests as $test_id => $test) {
    switch ($test['type']) {
      case PIFT_TYPE_FILE:
        $sorted['file'][$test_id] = $test;
        break;
      case PIFT_TYPE_BRANCH:
        $sorted['branch'][$test_id] = $test;
        break;
      default:
        break;
    }
  }
  return $sorted;
}
*/

/**
 * Check the 'testable' criteria for the specified issue.
 *
 * The checks are in order of expense.
 *
 * @param object $node
 *   The fully loaded issue node object.
 *
 * @return boolean
 *   Indicates whether the issue passes the criteria check.
 */
function pift_test_check_criteria_issue($node) {
  // Ensure we are passed an issue node
  if (!project_issue_node_is_issue($node)) {
    return FALSE;
  }
  // Ensure the issue has a valid 'testing' status
  if (!in_array($node->field_issue_status[LANGUAGE_NONE][0]['value'], variable_get('pift_status', array()))) {
    return FALSE;
  }
  // Ensure that the project has testing enabled.
  $pid = $node->field_project[LANGUAGE_NONE][0]['target_id'];
  if (!pift_project_enabled($pid)) {
    return FALSE;
  }
  // Ensure that one of the compatibility terms is present on the release node.
  $version = $node->field_issue_version[LANGUAGE_NONE][0]['value'];
  if (!($rid = project_release_exists($pid, $version))) {
    return FALSE;
  }
  $release_node = node_load($rid);
  $release_api = project_release_get_release_api_tid($release_node);
  if (array_key_exists($release_api, pift_core_api_versions())) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Check the 'testable' criteria for the specified file.
 *
 * @param object $file
 *   The loaded file object to check.
 *
 * @return boolean
 *   Indicates whether the file passes the criteria check.
 */
function pift_test_check_criteria_file($file) {
  if ((!isset($file->filename)) || !preg_match(PIFT_REGEX, $file->filename)) {
    return FALSE;
  }
  return TRUE;
}

/**
 * Retrieve the comment ID where a given test patch was added.
 *
 * @param array $test
 *   The test information array being queried.
 *
 * @result integer
 *   The associated comment ID from when the comment was added.
 */
function pift_test_get_comment_id($test) {
  if ($test['type'] == PIFT_TYPE_FILE) {
    $issue = node_load($test['nid']);
    // Get comment ID
    $file_changes = nodechanges_get_changed_cids_keyed($issue, 'field_issue_files', 'fid');
    return (isset($file_changes[$test['id']])) ? $file_changes[$test['id']] : 0;
  }
  return FALSE;
}

/**
 * Add files to test tables.
 *
 * @param array $files
 *   The loaded file objects to add.
 * @param integer $nid
 *   The id of the node the files are associated with.
 */
function pift_test_add_files($files, $nid) {
  foreach ($files as $fid => $file) {
    if (pift_test_check_criteria_file($file)) {
      pift_test_add(PIFT_TYPE_FILE, $file->fid, $nid);
    }
  }
}

/**
 * Add a test to the test tables.
 *
 * @param integer $type
 *   The type of test to add, PIFT_TYPE_BRANCH or PIFT_TYPE_FILE.
 * @param integer $id
 *   The vcs release label id for a branch test, or the file ID for a patch
 *   test, depending on test type.
 * @param integer $nid
 *   The ID of the project release node, for a branch test, or the issue node
 *   to which the file was attached, for a file test.
 */
function pift_test_add($type, $id, $nid) {
  $id = db_insert('pift_data')
    ->fields(array(
      'type' => $type,
      'id' => $id,
      'nid' => $nid,
      'status' => PIFT_STATUS_QUEUE,
    ))
    ->execute();
}

/**
 * Mark a test as sent.
 *
 * @param integer $test_id
 *   The unique identifier for this test.
 * @param integer $type
 *   The type of test, PIFT_TYPE_BRANCH or PIFT_TYPE_FILE.
 * @param integer $id
 *   The vcs release label id for a branch test, or the file ID for a patch
 *   test, depending on test type.
 */
function pift_test_sent($test_id, $type, $id) {
  $query = db_update('pift_data')
    ->fields(array(
      'test_id' => $test_id,
      'status' => PIFT_STATUS_SENT,
    ))
    ->condition('type', $type);
  if ($type == PIFT_TYPE_FILE) {
    $query->condition('id', $id);
  }
  elseif ($type == PIFT_TYPE_BRANCH) {
    $query->condition('nid', $id);
  }
  else {
    // This condition should never happen, but let's ensure that mistakes don't
    // cause us to inadvertently mark *all* tests as sent.
    return;
  }
  $query->execute();
}

/**
 * Add a completed test back to the testing queue.
 *
 * @param integer $test_id
 *    Unique test identifier for this test.
 */
function pift_test_requeue($test_id) {
  return db_update('pift_data')
    ->fields(array(
      'status' => PIFT_STATUS_QUEUE,
    ))
    ->condition('test_id', $test_id)
    ->execute();
}

/**
 * Deletes the specified test from the pift_data database table.
 *
 * @param integer $test_id
 *   Unique test identifier for this test.
 */
function pift_test_delete($test_id) {
  pift_test_delete_multiple(array($test_id));
}

/**
 * Deletes the specified tests from the {pift_data} db table.
 *
 * @param array $test_ids
 *   An array of unique test IDs.
 */
function pift_test_delete_multiple($test_ids) {
  db_delete('pift_data')
      ->condition('test_id', $test_ids, 'IN')
      ->execute();
}

/**
 * Deletes any file tests for the given file id from the {pift_data} db table.
 *
 * @param integer $fid
 *   The unique file id whose test results are to be removed.
 */
function pift_test_delete_file_test($fid) {
  db_delete('pift_data')
    ->condition('type', PIFT_TYPE_FILE)
    ->condition('id', $file->fid)
    ->execute();
}

/**
 * Deletes any file tests for the given issue nid.
 *
 * @param integer $nid
 *   The issue node id whose test results are to be removed.
 */
function pift_test_delete_issue_tests($nid) {
  db_delete('pift_data')
    ->condition('type', PIFT_TYPE_FILE)
    ->condition('nid', $nid)
    ->execute();
}

/**
 * Update a test record with the results from the PIFR server.
 *
 * @param array $result
 *   The results array received in the response from the PIFR server.
 */
function pift_test_update_result($result) {
  db_update('pift_data')
    ->fields(array(
        'status' => $result['pass'] ? PIFT_STATUS_PASS : PIFT_STATUS_FAIL,
        'message' => $result['message'],
        'last_tested' => REQUEST_TIME,
      ))
    ->condition('test_id', $result['test_id'])
    ->execute();
}
